#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <File.h>
#include <fs_attr.h>

#include "BF_Roster.h"
#include "BF_Msg.h"
#include "BF_GUI_Func.h"
#include "BF_GUI_Tools.h"
#include "BF_GUI_FilePanel.h"
#include "BF_GUI_KeysMenu.h"
#include "BF_GUI_Viewer.h"

int32 BF_GUI_FilePanel_MoveThread(void *data);

void
BF_GUI_FilePanel::Action_MoveFiles_0()
{
	/* disable panels */
	BF_GUI_Func_PanelsEnable(false);
	/* make message for dialog */
	BMessage oMessage(BF_MSG_MOVE);
	oMessage.AddPointer("bf_focus",this);
	/* make dialog */	
	float fWidth = poWinView->Bounds().Width();
	if(fWidth<400) fWidth-=40;else fWidth=400;
	BF_GUI_Dialog *poDialog = new BF_GUI_Dialog(BRect(30,30,fWidth,120),
		"Move file(s) to ","dialog",&oMessage,BG_GUI_DIALOG_WINRESIZE_MOVE_CENTER);
	poDialog->MoveToCenter( poWinView );	
	/* insert views */
	BRect oRect;
	/* insert text */
	poDialog->LocalBounds(oRect);
	/* insert edit_view */
	poDialog->LocalBounds(oRect);	
	//oRect.top = oRect.bottom+5;
	oRect.bottom = oRect.top+poSysSetup->oFontToolView.fHeight;
	BF_GUI_ViewEdit *poEdit = new BF_GUI_ViewEdit(oRect,"edit",
					poFriend->oPath.Path(),
					B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP,B_NAVIGABLE);
	poDialog->AddChild(poEdit);
	/* finish */
	poWinView->AddChild( poDialog );		
}


void
BF_GUI_FilePanel::Action_MoveFiles_1(BMessage *po_Message)
{
	/* check messahe */
	ASSERT(po_Message);
	bool bOk=FALSE;
	ASSERT(B_OK==po_Message->FindBool(BF_GUI_DIALOG_MESSAGE_DATA_OK,&bOk));
	if(!bOk){
		Action_MoveFiles_2(po_Message);
		return;
	}
	/* prepare new path */		
	char *pcPath=NULL;	
	{
		ASSERT(B_OK==po_Message->FindString("edit",(const char**)&pcPath) && pcPath);
		if(oPath.Compare(pcPath))
		{			
			Action_MoveFiles_2(po_Message);
			return;
		}
	}
	/* make new path */
	BF_Path * poNewPath = new BF_Path(pcPath);
	/* check volumes */
	{
		BL_String sVolFrom,sVolTo;		
		oPath.GetVolume(sVolFrom);
		poNewPath->GetVolume(sVolTo);
		/* volumes are not identifical :( */
		if(sVolFrom!=sVolTo){			
			delete poNewPath;
			Action_CopyFiles_1(po_Message,true);
			return;		
		}		
	}	
	/* get nodes */
	BF_NodeCollection *ploSelNode = Nodes_GetSelecting();	
	ASSERT(ploSelNode && ploSelNode->poPath);
	// make semaphor 
	bool *pbSemaphor = new bool;
	*pbSemaphor = false;
	/* prepare message for dialog */
	BMessage oMessageForDialog(BF_MSG_MOVE_FINISHED);
	oMessageForDialog.AddPointer("bf_focus",(void*)this);	
	oMessageForDialog.AddPointer("bf_semaphor",(void*)pbSemaphor);	
	// make status_dialog //
	BRect oRect(0,0,400,120);
	BF_GUI_Dialog * poPanel = new BF_GUI_Dialog(oRect,"Moving...",
		"progress_dialog",&oMessageForDialog,BG_GUI_DIALOG_WINRESIZE_MOVE_CENTER);
	poPanel->MoveToCenter(poWinView);
	// make status_view	
	poPanel->LocalBounds(oRect);
	oRect.top+=10;
	oRect.bottom = oRect.top+poSysSetup->oFontToolView.fHeight*2;	
	BF_GUI_ViewProgress *poStatus = new BF_GUI_ViewProgress(oRect,"progress",
		"moving files",B_FOLLOW_NONE,B_WILL_DRAW);
	poStatus->SetLimit(ploSelNode->CountItems());
	poPanel->AddChild(poStatus);	
	//
	BL_List *ploMenu = new BL_List();	
	ploMenu->AddItem(new BF_GUI_ViewMenu_Item("Cancel","cancel"));
	//
	poPanel->LocalBounds(oRect);
	oRect.bottom -= 5;
	oRect.top = oRect.bottom-poSysSetup->oFontToolView.fHeight;
	//
	BView * poMenu = new BF_GUI_ViewHMenu(oRect,"menu",
		B_FOLLOW_ALL,ploMenu);
	poPanel->AddChild(poMenu);
	// start dialog
	poWinView->AddChild(poPanel);
	poMenu->MakeFocus();
	///
	BMessage *poMsgForThread = new BMessage(BF_MSG_MOVE_FINISHED);
	poMsgForThread->AddPointer("bf_nodelist",(void*)ploSelNode);
	poMsgForThread->AddPointer("bf_focus",(void*)this);
	poMsgForThread->AddPointer("bf_dialog",(void*)poPanel);
	poMsgForThread->AddPointer("bf_semaphor",(void*)pbSemaphor);	
	poMsgForThread->AddPointer("bf_poPath",(void*)poNewPath);	
	thread_id idThreadMoving = spawn_thread(BF_GUI_FilePanel_MoveThread,"BeFar:moving",
		B_THREAD_SUSPENDED,(void*)poMsgForThread);	
	if(idThreadMoving<=0)	ASSERT("can`t start thread BeFar:moving\n");	
	ASSERT(B_OK==resume_thread(idThreadMoving));		
	//	
}

void
BF_GUI_FilePanel::Action_MoveFiles_2(BMessage *po_Message)
{
	// get done list from message	
	BF_Path *poNewPath = NULL;
	if(B_OK==po_Message->FindPointer("bf_poPath",(void**)&poNewPath) && poNewPath){
		delete poNewPath;
	}
	//	
	BF_GUI_Func_PanelsEnable(true);
	MakeFocus();
}


int32 
BF_GUI_FilePanel_MoveThread(void *data)
{
	ASSERT(data);
	//
	BMessage *poMessage = (BMessage*)data;
	//
	BF_NodeCollection *ploReqList=NULL;
	ASSERT(B_OK==poMessage->FindPointer("bf_nodelist",(void**)&ploReqList) && ploReqList);	
	BF_NodeCollection *ploDoneList = new BF_NodeCollection();
	bool 	*pbSemaphor;
	BF_Path	*poNewPath;
	ASSERT(B_OK==poMessage->FindPointer("bf_semaphor",(void**)&pbSemaphor) && pbSemaphor);		
	ASSERT(B_OK==poMessage->FindPointer("bf_poPath",(void**)&poNewPath) && poNewPath!=NULL);	
	//
	ASSERT(poSysKeysMenu);
	BView *poProgress = poSysKeysMenu->Window()->FindView("progress");
	ASSERT(poProgress);
	
	// start deleting files //
	{
		BMessenger oMessenger(poProgress);
		BMessage   oMessage(BF_MSG_PROGRESS_STEP);
		//
		BF_Node  	*poNode;
		BString		sPath(poNewPath->Path());
		bool		bRename = sPath.FindFirst('/')<0;
		status_t	uOk;
		//
		for(int i=0;i<ploReqList->CountItems();i++){
			poNode = (BF_Node*)ploReqList->ItemAt(i);
			ASSERT(ploReqList->poPath);	
			/* rename node */			
			if(bRename){				
				uOk = BF_Roster_RenameNode(*ploReqList->poPath,poNode,sPath.String());
				if(B_OK!=uOk) printf("ups...can`t rename node \n");
			}else
			// move node to new dir
			if(
				(BF_Roster_MoveNode(*ploReqList->poPath,poNode,poNewPath->Path()))
			)
			{
				ploDoneList->AddItem(poNode);
			}
			//
			//snooze(100);
			if(*pbSemaphor) break;
			oMessenger.SendMessage(&oMessage);
			//
		}
	}
	delete pbSemaphor;
	// send final message //
	{
		//
		// send message to progress_dialog about closed //
		{
			BView *poView=NULL;						
			ASSERT(B_OK==poMessage->FindPointer("bf_dialog",(void**)&poView) && poView);
			BMessenger	oMessenger(poView);
			BMessage 	oMessage(BF_MSG_DIALOG_CLOSE_NOW);
			oMessenger.SendMessage(&oMessage);
		}
		printf("BF_GUI_FilePanel_MoveThread send message \n");		
		// send message to panel about finishing //
		{		
			ploDoneList->poPath = poNewPath;
			BView *poView=NULL;
			ASSERT(B_OK==poMessage->FindPointer("bf_focus",(void**)&poView) && poView);
			BMessenger	oMessenger(poView);
			poMessage->ReplacePointer("bf_nodelist",ploDoneList);
			oMessenger.SendMessage(poMessage);
		}
	}	
	delete ploReqList;
	
	return 0;
}

