#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Clipboard.h>
//#include <time.h>
#include <NodeMonitor.h>
//#include <InputServerDevice.h>

#include "BF_Roster.h"
#include "BF_Msg.h"
#include "BF_GUI_FilePanel.h"
#include "BF_GUI_Tools.h"
#include "BF_GUI_KeysMenu.h"
#include "BF_GUI_Viewer.h"

#include "BF_GUI_InfoPanel.h"
#include "BF_GUI_AttrDialog.h"

#include "BF_BetaTesting.h"


#define INFO_SIZE	30 

BF_GUI_FilePanel::BF_GUI_FilePanel(
	BRect & o_Rect,
	BF_Panel_Pos	i_WinPosition)
:BF_GUI_Panel(o_Rect,"file",i_WinPosition) 
{
	poFriend = NULL;
	poInfoPanel = NULL;
	idThreadDeleting = 0;
	iPulseActions = BF_FILEPANEL_PULSE_NONE;
	/**/
	fFullName = 0;
	fFullSize = 0;
	fFullDate = 0;
	iNodesOnScreen = 0;
	iMode = BF_FILEPANEL_MODE_FULL;
	iModeFullStyles = BF_FILEPANEL_FULL_SIZE 
		| BF_FILEPANEL_FULL_SICON 
		| BF_FILEPANEL_FULL_NAME 
		| BF_FILEPANEL_FULL_DATE
		| BF_FILEPANEL_FULL_TYPE
		;
		
	fFileInfoHeight = 0;
	fFileInfoHeight = 0;
	/**/
	//if(BF_PANEL_WINPOS_LEFT!=i_WinPosition){
	{
		iMode = BF_FILEPANEL_MODE_COLS;	
		iModeColsStyles = BF_FILEPANEL_COLS_NAME;
		//iModeColsStyles = iModeColsStyles | BF_FILEPANEL_COLS_SICON;
		iColsColCount = 3;
	}
	/**/
	iSortType = /*BF_NODELIST_SORT_NAME*/BF_NODELIST_SORT_EXT;
	iSortStyle = BF_SORT_STYLE_DIRFIRST;
	iNavFirstIndex = 0;
	iNavCursorIndex = 0;
	sNodeSelMask = "*.*";
	bSelectDirAlso = false;
	/**/
	loNode.AddItem(new BF_Node("File1"));
	loNode.AddItem(new BF_Node("File2"));
	loNode.AddItem(new BF_Node("File3"));		
}

void					
BF_GUI_FilePanel::SetFriend(BF_GUI_FilePanel	*po_Friend)
{
	ASSERT(po_Friend);
	poFriend = po_Friend;
}

void
BF_GUI_FilePanel::AttachedToWindow(void)
{
	BView::AttachedToWindow();
	ASSERT(poFriend);
}


void					
BF_GUI_FilePanel::SetFlags(uint32 i_Mask)
{
	BView::SetFlags(i_Mask);
}

void					
BF_GUI_FilePanel::OnEnable(bool b_Enable)
{
	if(b_Enable){
		/* insert new nodes from  events_list */
		if(loNodeEvent.CountItems()>0){
			BF_NodeCollection_New(ploNode,oPath);
			loNodeEvent.NodesByType(ploNode,BF_NODE_EVENT_ADDED );
			Nodes_AddList(ploNode);
		}
		/* refresh nodes using events_list */
		if(loNodeEvent.CountItems()>0){
			BF_NodeCollection_New(ploNode,oPath);
			loNodeEvent.NodesByType(ploNode,BF_NODE_EVENT_CHANGED );
			Nodes_RefreshList(ploNode);
		}
		/* remove nodes using events_list */
		if(loNodeEvent.CountItems()>0){
			BF_NodeCollection_New(ploNode,oPath);
			loNodeEvent.NodesByType(ploNode,BF_NODE_EVENT_REMOVED );
			Nodes_RemoveList(ploNode);
		}
		/* clear events */
		loNodeEvent.DeleteItems();
	}else{
		PulseAction_Remove(BF_FILEPANEL_PULSE_ALL);
	}
}

void					
BF_GUI_FilePanel::Calc()
{
	SetFont(&poSysSetup->oFontNode.oFont,B_FONT_ALL);
	//
	fNodeHeight = (float)(int) ( poSysSetup->oFontNode.fHeight );	
	if(	ModeShowSIcon() && fNodeHeight<17) fNodeHeight = 17; 
	fFileInfoHeight = poSysSetup->oFontNode.fHeight;
	fSelInfoHeight = poSysSetup->oFontNode.fHeight;
	fColTitleHeight = poSysSetup->oFontNode.fHeight+4;
	//
	BRect oRect(Bounds());
	/* reserve place for path title */
	oRect.top += HeaderHeight();		
	oRect.top += fColTitleHeight;
	/* reserve place for bottom line,info line,selected line */
	oRect.bottom -= BottomHeight();
	oRect.bottom -= fFileInfoHeight;
	oRect.bottom -= fSelInfoHeight;
	fWidth = oRect.Width();
	/* reserve place for vertical lines */
	fWidth -= 6;
	/* calc total count of items */
	iNodesOnScreen = (int)( oRect.Height()/fNodeHeight );
	fNodesHeight = iNodesOnScreen * fNodeHeight;
	/* calc */
	switch(iMode){
	case BF_FILEPANEL_MODE_FULL:{
		fFullDate = 0;
		fFullSize = 0;
		fFullType = 0;	
		fFullSIcon = 0;	
		if(iModeFullStyles & BF_FILEPANEL_FULL_SICON) fFullSIcon=20;
		if(iModeFullStyles & BF_FILEPANEL_FULL_DATE)
			fFullDate= (float) (int ) ( poSysSetup->oFontNode.oFont.StringWidth("99-99-9999 99:99")+5 );
		if(iModeFullStyles & BF_FILEPANEL_FULL_SIZE)
			fFullSize=(float) (int ) ( poSysSetup->oFontNode.oFont.StringWidth("99999999999")+5 );
		fFullName = (float) (int ) ( fWidth -  fFullDate - fFullSize- fFullSIcon );
		if(iModeFullStyles & BF_FILEPANEL_FULL_TYPE){
			fFullType=(float) (int ) fFullName / 3;
			fFullName -= fFullType;
		}
		break;}
	case BF_FILEPANEL_MODE_COLS:{	
		fColsSIcon = 0;
		fColsColWidth = (float)(int) ( fWidth/iColsColCount );
		if(iModeColsStyles & BF_FILEPANEL_COLS_SICON)	fColsSIcon=20;
		fColsName = (float)(int) (fColsColWidth - fColsSIcon-6);
		
		iColsNodesInCol = iNodesOnScreen;
		iNodesOnScreen = iColsNodesInCol*iColsColCount;
		
		break;}
	}		
}

void 
BF_GUI_FilePanel::MakeNodesList()
{	
	loNodeEvent.DeleteItems();

	BF_NodeCollection_New(ploNode,oPath);
	ploNode->AddList(&loNode);
	Nodes_RemoveList(ploNode);

	int32 iLoadStyles = BF_ROSTER_LOAD_NODE_ALL;
	iLoadStyles = iLoadStyles | (((iMode==BF_FILEPANEL_MODE_FULL)  && (iModeFullStyles & BF_FILEPANEL_FULL_TYPE))?BF_ROSTER_LOAD_TYPE:0);
	iLoadStyles = iLoadStyles | (ModeShowSIcon()?BF_ROSTER_LOAD_SICON:0);

	BF_Roster_LoadNodeList(oPath,loNode,iLoadStyles	);
	iNavFirstIndex = 0;
	iNavCursorIndex = 0;
	loNode.Sort(iSortType,iSortStyle);	
}


void
BF_GUI_FilePanel::DrawStringRight(const char *pc_Text,BPoint &o_Point,float f_Width)
{
	float f = poSysSetup->oFontNode.oFont.StringWidth(pc_Text);
	poRender->MovePenTo(o_Point.x+f_Width-f,o_Point.y);
	poRender->DrawString(pc_Text);
}

void
BF_GUI_FilePanel::DrawStringCenter(const char *pc_Text,BPoint &o_Point,float f_Width)
{
	float f = poSysSetup->oFontNode.oFont.StringWidth(pc_Text);
	poRender->MovePenTo(o_Point.x+f_Width/2-f/2,o_Point.y);
	poRender->DrawString(pc_Text);
}

/*
bool 
BF_GUI_FilePanel::CheckStringWidth(BString & s_Text,float f_Width,char *pc_NewValue)
{
	 if(poSysSetup->oFontNode.oFont.StringWidth(s_Text.String())<=f_Width) return(true);
	 
	 bool bSelfNewValue = !pc_NewValue;
	 if(bSelfNewValue) pc_NewValue = (char*)malloc(s_Text.Length()+5);
	 
	 
	 float f;
	 strcpy(pc_NewValue,s_Text.String());
	 int iLen = strlen(pc_NewValue);
	 while(true){
	 	if(iLen<=0) break;
	 	pc_NewValue[--iLen]='\0';
	 	f = poSysSetup->oFontNode.oFont.StringWidth(pc_NewValue);
	 	if(f<=f_Width) break;
	 }	 
	 iLen = strlen(pc_NewValue);
	 if(iLen>2) pc_NewValue[iLen-1]='.';	 
 	 if(iLen>1) pc_NewValue[iLen-2]='.';
	 
	 if(bSelfNewValue){
	 	s_Text = pc_NewValue;
	 	free(pc_NewValue);
	 }		 
	return(false);
}
*/

void					
BF_GUI_FilePanel::DrawPanel(BRect & o_Rect)
{
	BF_GUI_Panel::DrawPanel(o_Rect);
	/* draw bottom_info */
	DrawListInfo(false);
	/* draw selected bottom_info */
	DrawSelListInfo(false);	
	/* draw verticals lines */	
	DrawLines(false);
	/* draw path */	
	DrawHeader(IsFocus());
	/* draw headers */	
	DrawColTitles();
	/* draw fileinfo */
	DrawFileInfo();
	/* draw items */
	DrawNodes();
}

void 
BF_GUI_FilePanel::DrawLines(bool b_OnlyNodes)
{
	/**/	
	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{	
		if(poSysSetup->iFPanelColsStyle & BF_SETUP_FPANELMODE_COLS_DRAWLINES){
			BRect oRect = poRender->Bounds();
			oRect.top=HeaderHeight(); 			
			oRect.bottom -= BottomHeight();
			oRect.bottom -= fFileInfoHeight;
			oRect.bottom -= fSelInfoHeight;
			oRect.left+=4;
			oRect.left--;
			poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_FRAME]);	
			for(uint32 i=0;i<iColsColCount-1;i++){
				oRect.left+=fColsColWidth;
				oRect.right = oRect.left;				
				poRender->StrokeRect(oRect);
			}
		}
		break;}
	case BF_FILEPANEL_MODE_FULL:{
		if(poSysSetup->iFPanelFullStyle & BF_SETUP_FPANELMODE_FULL_DRAWLINES){
			poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_FRAME]);	
			BRect oRect = poRender->Bounds();
			oRect.top=HeaderHeight(); 			
			oRect.bottom -= BottomHeight();
			oRect.bottom -= fFileInfoHeight;
			oRect.bottom -= fSelInfoHeight;
			oRect.left+=4;
			//
			if(iModeFullStyles & BF_FILEPANEL_FULL_SICON)	oRect.left+=fFullSIcon;
			oRect.left+=fFullName;
			/* line |type */				
			if(iModeFullStyles & BF_FILEPANEL_FULL_TYPE){						
				oRect.right = oRect.left;
				poRender->StrokeRect(oRect);
				oRect.left+=fFullType;
			}			
			/* line |size */				
			if(iModeFullStyles & BF_FILEPANEL_FULL_SIZE){
				oRect.right = oRect.left;
				poRender->StrokeRect(oRect);
				oRect.left+=fFullSize;
			}						
			/* line |date */
			if(iModeFullStyles & BF_FILEPANEL_FULL_DATE){						
				oRect.right = oRect.left;
				poRender->StrokeRect(oRect);		
				oRect.left+=fFullDate;
			}
		}
		break;}
	};
}

void 
BF_GUI_FilePanel::DrawListInfo(bool b_DrawBack)
{
	/* make info */
	BL_String s;
	s<<loNode.SizeAll(false);	
	s.SetDigits();
	s<<" bytes in ";
	s<<loNode.CountRealNodes();
	s<<" files";	
	DrawBottom(s.String(), b_DrawBack);
}

void
BF_GUI_FilePanel::DrawFileInfo()
{
	BRect oRect;
	/* erase background  */
	oRect = poRender->Bounds();
	oRect.left+=6;
	oRect.right-=6;
	oRect.bottom -= BottomHeight();
	oRect.top = oRect.bottom - fFileInfoHeight;
	poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_BACK]);			
	poRender->FillRect(oRect);	
	/* draw text */	
	BPoint 		o;	
	BL_String 	s;	
	
	o.Set(6,oRect.bottom-poSysSetup->oFontNode.fDescent);
	poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_PANELINFO_TEXT]);	
	poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);

	/* draw size */
	BF_Node 	*poNode = loNode.NodeAt(iNavCursorIndex);		
	if(poNode){
		switch(poNode->iType){
		case BF_NODE_TYPE_PARENTDIR:
			poRender->DrawString("..",o);	
			break;
		case BF_NODE_TYPE_DIR:
			//s=poNode->sName;
			//CheckStringWidth(s,oRect.Width()-12,NULL);
			DrawStringCheck(poNode->sName.String(),oRect.Width()-12,o);	
			break;
		case BF_NODE_TYPE_FILE:
			poRender->DrawString("Size : ",o);
			s = poNode->iSize;
			s.SetDigits();
			DrawStringRight(s.String(),o,oRect.Width()-12);
			break;
		}
	}
		
}

void
BF_GUI_FilePanel::DrawSelListInfo(bool b_DrawBack)
{
	BRect oRect(poRender->Bounds());
	
	oRect.left+=4;
	oRect.right-=4;		
	oRect.bottom -= BottomHeight() + fFileInfoHeight;
	oRect.top = oRect.bottom - fSelInfoHeight;	
	/* erase background  */
	if(b_DrawBack){						
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_BACK]);			
		poRender->FillRect(oRect);
	}
	/* make info */
	//oRect = poRender->Bounds();
	BL_String 	s;
	int32		iBytes,iCount;		
	float 		fTop = oRect.top + fSelInfoHeight/2;
	BPoint 		o1,o2;
	/* get info */
	/* draw */
	loNode.SelectedInfo(iBytes,iCount);
	if(iCount){
		s<<iBytes;	
		s.SetDigits();
		s<<" bytes in ";
		s<<iCount;
		s<<" files";		
	    float fWidth = poSysSetup->oFontNode.oFont.StringWidth(s.String());		
		float fLeft = (oRect.Width() - fWidth)/2;
		/* draw lines */		
		/* left line */
		o1.Set(4,fTop-1);
		o2.Set(fLeft-3,o1.y);
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_FRAME]);
		poRender->StrokeLine(o1,o2);	
		/* right line */
		o1.Set(fLeft+fWidth+3,fTop+1);
		o2.Set(oRect.right-3,o1.y);	
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_FRAME]);
		poRender->StrokeLine(o1,o2);	
		/* draw info */
		o1.Set(fLeft,oRect.top + poSysSetup->oFontNode.fAscent);
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_NODE_FOCUSSELECT]);
		poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);
		poRender->DrawString(s.String(),o1);
	}else{		
		/* full lines */
		o1.Set(4,fTop);
		o2.Set(oRect.right-3,o1.y);
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_FRAME]);
		poRender->StrokeLine(o1,o2);	
	}
}

void 
BF_GUI_FilePanel::DrawColTitles()
{	
	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{	
		BPoint		oPoint(7,HeaderHeight());
		oPoint.y += poSysSetup->oFontNode.fAscent;
		/* print headers */
		/* set colors */
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_HEADER_TEXT]);	
		poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);			
		oPoint.x = 5;
		for(uint i=0;i<iColsColCount;i++){
			/* header 'SmallIcon' */
			if(iModeColsStyles & BF_FILEPANEL_COLS_SICON) 	oPoint.x+=fColsSIcon;
			/* header 'Name' */
			DrawStringCenter("Name",oPoint,fColsName);			
			oPoint.x+=fColsName;
		}
		break;}
	case BF_FILEPANEL_MODE_FULL:{
		BPoint		oPoint(4,HeaderHeight());		
		/* start  */		
		oPoint.y += poSysSetup->oFontNode.fAscent;
		/*---------------*/		
		/* print headers */
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_HEADER_TEXT]);	
		poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);	
		oPoint.x = 4;
		/* header 'SmallIcon' */
		if(iModeFullStyles & BF_FILEPANEL_FULL_SICON){	
			//DrawStringCenter("Icon",oPoint,fFullSIcon);			
			oPoint.x+=fFullSIcon;
		}
		/* header 'Name' */
		DrawStringCenter("Name",oPoint,fFullName);			
		oPoint.x+=fFullName;
		/* header 'type' */
		if(iModeFullStyles & BF_FILEPANEL_FULL_TYPE){	
			DrawStringCenter("Type",oPoint,fFullType);			
			oPoint.x+=fFullType;			
		}		
		/* header 'Size' */
		if(iModeFullStyles & BF_FILEPANEL_FULL_SIZE){	
			DrawStringCenter("Size",oPoint,fFullSize);			
			oPoint.x+=fFullSize;			
		}	
		/* header 'Date' */
		if(iModeFullStyles & BF_FILEPANEL_FULL_DATE){
			DrawStringCenter("Date",oPoint,fFullDate);
			oPoint.x+=fFullDate;
		}
		/**/	
		break;}
	}
}

void 
BF_GUI_FilePanel::DrawNodes(bool b_DrawBackground)
{
	/* open transaction */
	Draw_OpenTransaction();
	
	BRect oRect(poRender->Bounds());
	oRect.left = 6;
	oRect.right -= 6;	
	oRect.top = HeaderHeight() + fColTitleHeight;
	oRect.bottom = oRect.top + fNodesHeight;
		
	if(b_DrawBackground)
	{				
		poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_BACK]);
		poRender->FillRect(oRect);		
		DrawLines( true );
	}
	

	/* draw  nodes */
	if(true)
	{
		BPoint		oPoint(7,HeaderHeight());	
		BRect		oRect;
		bool		bShowFocus = IsFocus() || bFixedFocus;			
		char		pc[500];
		int			iRealIndex=0;
		int			iLastNode=0;
		
		
		iLastNode = iNavFirstIndex + iNodesOnScreen;
				
		if(iLastNode>loNode.CountItems()) iLastNode = loNode.CountItems();		
		/* scan nodes */			
		for(iRealIndex=iNavFirstIndex;iRealIndex<iLastNode;iRealIndex++)
		{		
			DrawNode(iRealIndex,bShowFocus && (iRealIndex==iNavCursorIndex),oPoint,oRect,pc,NULL,false);
		}
		iRealIndex  = iRealIndex;
	}	
	if(b_DrawBackground){
	}
	/* close trans */
	Draw_CloseTransaction(&oRect);
}

void
BF_GUI_FilePanel::DrawNodeAt(int32 i_Index)
{
	if(i_Index<0) return;
	///
	BRect 	oRect,oRect2;
	BPoint	oPoint;
	char	pc[500];
	bool	bFocused=i_Index==iNavCursorIndex && (IsFocus() || bFixedFocus);
	
	DrawNode(i_Index,bFocused,oPoint,oRect,pc,&oRect2,true);
}

void 
BF_GUI_FilePanel::DrawNode(
	int i_NodeIndex,
	bool b_ItCursor,
	BPoint &oPoint,
	BRect & oRect,
	char *pc_Buffer,
	BRect *po_RectBound,
	bool b_DrawBackground,
	bool b_UsePoint)
{	
	/* draw in full_mode */
	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{
		BF_Node *poNode = (BF_Node*)loNode.NodeAt(i_NodeIndex);
		if(!poNode) return;
		
		uint32 iColIndex = (i_NodeIndex - iNavFirstIndex)/iColsNodesInCol;
		i_NodeIndex -= iColIndex*iColsNodesInCol;
				
		if(!b_UsePoint){
			oPoint.x = 4;
			oPoint.x += iColIndex * fColsColWidth;
			oPoint.y = HeaderHeight() + fColTitleHeight + poSysSetup->oFontNode.fAscent;
			oPoint.y += fNodeHeight * (i_NodeIndex - iNavFirstIndex);		
		}
		
		if(po_RectBound){
			po_RectBound->left = oPoint.x;
			po_RectBound->right  = po_RectBound->left + fColsColWidth-12;
			po_RectBound->top = oPoint.y - poSysSetup->oFontNode.fAscent;
			po_RectBound->bottom = po_RectBound->top + fNodeHeight;
		}		
		/* set color */	
		DrawNode_SetNodeColor(poNode,false);		
		/* draw node */					
		{		
			oPoint.x+=2;
			if(b_ItCursor)			DrawNode_CursorBack(poNode,oRect,oPoint,fColsColWidth-6,true);else 
			if(b_DrawBackground) 	DrawNode_CursorBack(poNode,oRect,oPoint,fColsColWidth-6,false);
			oPoint.x+=2;
			/* print small_icon */
			if(iModeColsStyles & BF_FILEPANEL_COLS_SICON){							
				if(poNode->poSIcon){
					poRender->SetDrawingMode(B_OP_ALPHA);		
					oPoint.y-=poSysSetup->oFontNode.fAscent;
					poRender->DrawBitmap(poNode->poSIcon,oPoint);
					oPoint.y+=poSysSetup->oFontNode.fAscent;
					poRender->SetDrawingMode(B_OP_COPY);
				}
				oPoint.x+=fColsSIcon;
			}
			/* print node_name */
			DrawStringCheck(poNode->sName.String(),fColsName-3,oPoint);
			
			/* for link print dot */
			if(poNode->bLink){
				BPoint o1(oPoint);
				o1.y += poSysSetup->oFontNode.fDescent-1;
				float fx1 = oPoint.x + poSysSetup->oFontNode.oFont.StringWidth(poNode->sName.String());
				for(float fx = oPoint.x;fx<fx1;fx+=2.0){			
					StrokeLine(o1,o1);
					o1.x+=2.0;
				}
			}

			oPoint.x+=fColsColWidth;
		}		
		break;}
	case BF_FILEPANEL_MODE_FULL:{
		oPoint.x = 4;
		oPoint.y = HeaderHeight() + fColTitleHeight + poSysSetup->oFontNode.fAscent;
		oPoint.y += fNodeHeight * (i_NodeIndex - iNavFirstIndex);
		if(po_RectBound){
			po_RectBound->left = oPoint.x;
			po_RectBound->right  = po_RectBound->left + fWidth-12;
			po_RectBound->top = oPoint.y - poSysSetup->oFontNode.fAscent;
			po_RectBound->bottom = po_RectBound->top + fNodeHeight;
		}
		BF_Node *poNode = (BF_Node*)loNode.NodeAt(i_NodeIndex);
		if(!poNode) return;
		/* set color */	
		DrawNode_SetNodeColor(poNode,false);
		/* draw node */					
		{		
			oPoint.x+=2;
			/* print small_icon */						
			if(b_ItCursor)			DrawNode_CursorBack(poNode,oRect,oPoint,fFullSIcon + fFullName-5,true);else 
			if(b_DrawBackground) 	DrawNode_CursorBack(poNode,oRect,oPoint,fFullSIcon + fFullName-5,false);			
			oPoint.x+=2;

			/* for link print dot */
			if(poNode->bLink){
				oPoint.x-=1;
				poRender->DrawString(".",oPoint);
				oPoint.x+=1;
			}						

		
			if(iModeFullStyles & BF_FILEPANEL_FULL_SICON){				
				if(poNode->poSIcon){
					poRender->SetDrawingMode(B_OP_ALPHA);		
					oPoint.y-=poSysSetup->oFontNode.fAscent;
					poRender->DrawBitmap(poNode->poSIcon,oPoint);
					oPoint.y+=poSysSetup->oFontNode.fAscent;
					poRender->SetDrawingMode(B_OP_COPY);
				}
				oPoint.x+=fFullSIcon;
			}
			/* print node_name */
			DrawStringCheck(poNode->sName.String(),fFullName-8,oPoint);

			oPoint.x+=fFullName-4;
			/* print node_type */
			if(iModeFullStyles & BF_FILEPANEL_FULL_TYPE){
				oPoint.x+=2;
				
				if(b_ItCursor)			DrawNode_CursorBack(poNode,oRect,oPoint,fFullType-5,true);else
				if(b_DrawBackground) 	DrawNode_CursorBack(poNode,oRect,oPoint,fFullType-5,false);
				
				DrawStringCheck(poNode->sType.String(),fFullType-5,oPoint);
				/*
				if(CheckStringWidth(poNode->sType,fFullType-5,pc_Buffer)){
					poRender->DrawString(poNode->sType.String(),oPoint);
				}else{
					poRender->DrawString(pc_Buffer,oPoint);
				}*/
				//DrawString(poNode->sType.String(),oPoint);
				oPoint.x+=fFullType-2;
			}				
			/* print node_size */
			if(iModeFullStyles & BF_FILEPANEL_FULL_SIZE){
				oPoint.x+=2;
				if(b_ItCursor)			DrawNode_CursorBack(poNode,oRect,oPoint,fFullSize-5,true);else
				if(b_DrawBackground) 	DrawNode_CursorBack(poNode,oRect,oPoint,fFullSize-5,false);
				switch(poNode->iType){
				case BF_NODE_TYPE_FILE:{
					// this is not debug print !!! //
					sprintf(pc_Buffer,"%i",(int)poNode->iSize);	
					DrawStringRight(pc_Buffer,oPoint,fFullSize-5);
					break;}
				case BF_NODE_TYPE_DIR:{
					DrawStringRight("<dir>",oPoint,fFullSize-5);
					break;}
				};
				//if(poNode->bLink) poRender->DrawString(".",oPoint);
				oPoint.x+=fFullSize-2;
			}
			/* print node_date */
			if(iModeFullStyles & BF_FILEPANEL_FULL_DATE){
				oPoint.x+=2;
				if(b_ItCursor)			DrawNode_CursorBack(poNode,oRect,oPoint,fFullDate-5,true);else
				if(b_DrawBackground) 	DrawNode_CursorBack(poNode,oRect,oPoint,fFullDate-5,false);
				if(poNode->iType!=BF_NODE_TYPE_PARENTDIR){
					strcpy(pc_Buffer,"");
					if(poNode->iType!=BF_NODE_TYPE_PARENTDIR){
						//printf("name %s time %i\n",poNode->sName.String(),poNode->uCreateTime);												
						tm* puTime = localtime(&poNode->uCreateTime);
						char pc1[10];
						ASSERT(puTime);												
						strcpy(pc_Buffer,"");					
						//
						sprintf(pc1,"%i",puTime->tm_mday);
						if(puTime->tm_mday<10) strcat(pc_Buffer,"0");
						strcat(pc_Buffer,pc1);
						strcat(pc_Buffer,".");
						//
						sprintf(pc1,"%i",puTime->tm_mon+1);
						if(puTime->tm_mon<9) strcat(pc_Buffer,"0");
						strcat(pc_Buffer,pc1);					
						strcat(pc_Buffer,".");
						//
						sprintf(pc1,"%i",puTime->tm_year+1900);					
						strcat(pc_Buffer,pc1);					
						strcat(pc_Buffer," ");
						//
						sprintf(pc1,"%i",puTime->tm_hour);
						if(puTime->tm_hour<10) strcat(pc_Buffer,"0");
						strcat(pc_Buffer,pc1);
						strcat(pc_Buffer,":");
						//
						sprintf(pc1,"%i",puTime->tm_min);
						if(puTime->tm_min<10) strcat(pc_Buffer,"0");
						strcat(pc_Buffer,pc1);					
																										
						//sprintf(pc_Buffer,"%i",poNode->uCreateTime);
					}			
					DrawStringRight(pc_Buffer,oPoint,fFullDate-5);
				}
				oPoint.x+=fFullDate-2;
			}
		}
		break;};
	}
}

void 					
BF_GUI_FilePanel::DrawNodeSelf( /* wrapper for prev method  */
	int i_NodeIndex,
	bool b_ItCursor,										
	bool b_DrawBackground,
	bool b_OffScreen)
{
	ASSERT(! b_OffScreen);
	
	char	pcBuffer[500];
	BPoint 	oPoint;
	BRect  	oRect;
	if(!b_OffScreen){
		DrawNode(i_NodeIndex,b_ItCursor,oPoint,oRect,pcBuffer,NULL,b_DrawBackground);
	}else{
	/*
		switch(iMode){
		case BF_FILEPANEL_MODE_COLS:{
			BRect oRectOld(0,0,fColsColWidth-1,fNodeHeight-1),oRect;
			BRect oRectNew;
			
			oRectNew.left = 4;
			oRectNew.left += ((i_NodeIndex - iNavFirstIndex)/iColsNodesInCol) * fColsColWidth;
			oRectNew.right = oRectNew.left + oRectOld.Width();
			oRectNew.top = HeaderHeight() + fColTitleHeight;
			oRectNew.top += fNodeHeight * (i_NodeIndex - iNavFirstIndex);		
			oRectNew.bottom = oRectNew.top + oRectOld.Height();
			
			oPoint.x = 0;
			oPoint.y = poSysSetup->oFontNode.fAscent;
			
			
			poRenderBitmap = new BBitmap(oRectOld,B_RGB16,true);		
			poRenderBitmap->AddChild( poRender = new BView(oRectOld,"",B_FOLLOW_ALL,B_WILL_DRAW) );
			poRenderBitmap->Lock();
			poRender->SetViewColor(B_TRANSPARENT_COLOR);						
			
			oRect = oRectOld;
			DrawNode(i_NodeIndex,b_ItCursor,oPoint,oRect,pcBuffer,NULL,true,true);
			
			poRender->Sync();
			poRenderBitmap->Unlock();
			
			DrawBitmap(poRenderBitmap,oRectOld,oRectNew);
								
			delete poRenderBitmap;		
			
			poRenderBitmap = NULL;
			poRender = this;	
			
			break;}			
		default:
			ASSERT(false);
		}
		*/
	}
}

void 
BF_GUI_FilePanel::DrawNode_SetNodeColor(BF_Node *po_Node,bool b_IsFocus)
{
	if(b_IsFocus){
		if(po_Node && po_Node->bSelect)	poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_NODE_FOCUSSELECT]);
		else							poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_CURSOR_TEXT]);
		poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_CURSOR_BACK]);
	}else{
		if(po_Node && po_Node->bSelect){
			poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_NODE_SELECT]);		
			poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);
		}else{
			switch(po_Node?po_Node->iType:BF_NODE_TYPE_UNKNOWN){
			case BF_NODE_TYPE_DIR:
				poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_NODE_DIR]);		
				poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);
				break;
			default:
				poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_NODE]);					
				poRender->SetLowColor(poSysSetup->aColor[BF_COLOR_BACK]);	
			}
		}
	}
}

void
BF_GUI_FilePanel::DrawNode_CursorBack(
	BF_Node *po_Node
	,BRect &oRect
	,BPoint &oPoint
	,float f_Width
	,bool b_Cursor)
{
	oRect.left=oPoint.x;
	oRect.right=oRect.left+f_Width;
	oRect.top = oPoint.y - poSysSetup->oFontNode.fAscent;
	oRect.bottom = oRect.top+fNodeHeight-1;	
	if(b_Cursor)	poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_CURSOR_BACK]);		
	else			poRender->SetHighColor(poSysSetup->aColor[BF_COLOR_BACK]);
	poRender->FillRect(oRect);
	//
	DrawNode_SetNodeColor(po_Node,b_Cursor);
}


void
BF_GUI_FilePanel::MouseUp(BPoint point)
{
	if(iPulseActions & BF_FILEPANEL_PULSE_MOVEDOWN){
		PulseAction_Remove(BF_FILEPANEL_PULSE_MOVEDOWN);
	}
	if(iPulseActions & BF_FILEPANEL_PULSE_MOVEUP){
		PulseAction_Remove(BF_FILEPANEL_PULSE_MOVEUP);
	}
}

void					
BF_GUI_FilePanel::MouseDown(BPoint point)
{
	/* check for disable */
	if(!Enabled()) return;	
	/* check for pressing in path_line */
	if(point.y<=HeaderHeight()){
		Action_SelVolume_0(iWinPosition);
		return;
	}	
	/* check for focus */
	if(!IsFocus()){
		MakeFocus();
	}
	/* check for pressing in colstitle area */		
	if(point.y<=(fColTitleHeight+HeaderHeight())){
		NavUp();
		PulseAction_Add(BF_FILEPANEL_PULSE_MOVEUP);
		return;
	}	
	/* check for pressing in under_nodes area */		
	if(point.y>(fColTitleHeight+HeaderHeight()+fNodesHeight)){
		NavDown();
		PulseAction_Add(BF_FILEPANEL_PULSE_MOVEDOWN);
		return;
	}		
	/*  */	
	int32 	iNewCursor=-1;	
	BPoint 	oPoint(point);
		
	oPoint.y -= fColTitleHeight+HeaderHeight();
	
	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{
		//oPoint.y -= poSysSetup->oFontNode.fAscent + fNodeHeight;
		iNewCursor = (int32)(oPoint.y/fNodeHeight) + iNavFirstIndex;
		iNewCursor+=iColsNodesInCol* int32((point.x-5)/fColsColWidth);
		break;}	
	case BF_FILEPANEL_MODE_FULL:{		
		//oPoint.y -= poSysSetup->oFontNode.fAscent + fNodeHeight;
		iNewCursor = (uint)(oPoint.y/fNodeHeight) + iNavFirstIndex;				
		break;}
	};
	int32 iNum;
	Window()->CurrentMessage()->FindInt32("clicks", &iNum);		
	if (iNum==2)
		NavEnter();			
	else
	if(iNewCursor==iNavCursorIndex)		
		NavSelect(false);
	else
		NavChangeCursor(iNewCursor);
}

void					
BF_GUI_FilePanel::Hide()
{
	BF_GUI_Panel::Hide();
	if(poInfoPanel) poInfoPanel->Hide();
}
void					
BF_GUI_FilePanel::Show()
{
	BF_GUI_Panel::Show();
	if(poInfoPanel) poInfoPanel->Show();
}

void
BF_GUI_FilePanel::PrepareKeysMenu()
{
	BF_GUI_Panel::PrepareKeysMenu();
	uint32 	iKeysModifiers = modifiers();
	//printf("iModifiers=%i,%i\n",iKeysModifiers,iKeysModifiers & B_RIGHT_OPTION_KEY);
	if(iKeysModifiers & B_RIGHT_OPTION_KEY){
		BMessage *po;
		
		poSysKeysMenu->SetText(1,"SrtName");
		po = new BMessage(BF_MSG_FILEPANEL_SORT);
		po->AddInt32("bf_sorttype",BF_NODELIST_SORT_NAME);		
		poSysKeysMenu->SetMessage(1,po,this);
		
		poSysKeysMenu->SetText(2,"SrtSize");
		po = new BMessage(BF_MSG_FILEPANEL_SORT);
		po->AddInt32("bf_sorttype",BF_NODELIST_SORT_SIZE);		
		poSysKeysMenu->SetMessage(2,po,this);
		
		poSysKeysMenu->SetText(3,"SrtDate");
		po = new BMessage(BF_MSG_FILEPANEL_SORT);
		po->AddInt32("bf_sorttype",BF_NODELIST_SORT_DATE);		
		poSysKeysMenu->SetMessage(3,po,this);
		
		poSysKeysMenu->SetText(4,"SrtExt");
		po = new BMessage(BF_MSG_FILEPANEL_SORT);
		po->AddInt32("bf_sorttype",BF_NODELIST_SORT_EXT);		
		poSysKeysMenu->SetMessage(4,po,this);
		
		poSysKeysMenu->SetText(5,"MakeLink");
		poSysKeysMenu->SetMessage(5,new BMessage(BF_MSG_FILEPANEL_MAKELINK_START),this);				

		//poSysKeysMenu->SetText(7,"Search");
		poSysKeysMenu->SetText(8,"To Void");
		poSysKeysMenu->SetMessage(8,new BMessage(BF_MSG_FILEPANEL_MOVETOVOID_START),this);		
	}else
	{
		poSysKeysMenu->SetText(1,"LeftVol");
		poSysKeysMenu->SetMessage(1,new BMessage(BF_MSG_FILEPANEL_VOL_LEFT_START),this);
		poSysKeysMenu->SetText(2,"RightVol");		
		poSysKeysMenu->SetMessage(2,new BMessage(BF_MSG_FILEPANEL_VOL_RIGHT_START),this);
		poSysKeysMenu->SetText(3,"View");
		poSysKeysMenu->SetMessage(3,new BMessage(BF_MSG_FILEPANEL_VIEWER_START),this);
		poSysKeysMenu->SetText(5,"Copy");
		poSysKeysMenu->SetMessage(5,new BMessage(BF_MSG_FILEPANEL_COPY_START),this);
		poSysKeysMenu->SetText(6,"RenMove");
		poSysKeysMenu->SetMessage(6,new BMessage(BF_MSG_FILEPANEL_MOVE_START),this);
		poSysKeysMenu->SetText(7,"MakeDir");		
		poSysKeysMenu->SetMessage(7,new BMessage(BF_MSG_FILEPANEL_MAKEDIR_START),this);
		poSysKeysMenu->SetText(8,"To Trash");	
		poSysKeysMenu->SetMessage(8,new BMessage(BF_MSG_FILEPANEL_MOVETOTRASH_START),this);
		poSysKeysMenu->SetText(10,"Exit");
		poSysKeysMenu->SetMessage(10,new BMessage(BF_MSG_MAIINVIEW_ASK_QUIT),poWinView);
	}
}

void					
BF_GUI_FilePanel::KeyDown(const char *bytes, int32 numBytes)
{
	uint32 	iKeysModifiers = modifiers();
	if(numBytes==1 && bytes[0]==B_BACKSPACE){		
		NavGoParentDir();
	}else	
	if(numBytes==1 && bytes[0]==B_ENTER){
		NavEnter();
	}else
	if(numBytes==1 && bytes[0]==B_DOWN_ARROW){
		NavDown();
	}else
	if(numBytes==1 && bytes[0]==B_UP_ARROW){
		NavUp();
	}else
	if(numBytes==1 && bytes[0]==B_RIGHT_ARROW && (BF_FILEPANEL_MODE_COLS==iMode)){
		NavRight();		
	}else
	if(numBytes==1 && bytes[0]==B_LEFT_ARROW && (BF_FILEPANEL_MODE_COLS==iMode)){
		NavLeft();
	}else	
	if(numBytes==1 && bytes[0]==B_UP_ARROW){
		NavUp();
	}else	
	if(numBytes==1 && bytes[0]==B_HOME){
		NavToStart();
	}else
	if(numBytes==1 && bytes[0]==B_END){
		NavToEnd();
	}else	
	if(numBytes==1 && bytes[0]==B_INSERT){
		NavSelect();
	}else		
	if(numBytes==1 && bytes[0]==B_DELETE){
		Action_DeleteFiles_0(BF_FILEPANEL_REMOVE_TO_TRASH);
	}else			
	if(numBytes==1 && bytes[0]==B_PAGE_UP){
		NavPgUp();
	}else			
	if(numBytes==1 && bytes[0]==B_PAGE_DOWN){
		NavPgDown();
	}else				
	if(bytes[0] == B_FUNCTION_KEY){
		int32 iKey;
		BMessage *poMsg = Window()->CurrentMessage();
		if(!poMsg) return;
		poMsg->FindInt32("key", &iKey);
		if(iKeysModifiers & B_RIGHT_OPTION_KEY){
			switch(iKey){
			case B_F1_KEY:
				ChangeSortType(BF_NODELIST_SORT_NAME);
				break;
			case B_F2_KEY:
				ChangeSortType(BF_NODELIST_SORT_SIZE);
				break;
			case B_F3_KEY:
				ChangeSortType(BF_NODELIST_SORT_DATE);
				break;				
			case B_F4_KEY:
				ChangeSortType(BF_NODELIST_SORT_EXT);
				break;				
			case B_F5_KEY:
				Action_MakeLink_0();
				break;
			case B_F8_KEY:
				Action_DeleteFiles_0(BF_FILEPANEL_REMOVE_TO_VOID);
				break;
			};	
		}else{
			switch(iKey){
			case B_F1_KEY:
				Action_SelVolume_0(BF_PANEL_WINPOS_LEFT);
				break;
			case B_F2_KEY:
				Action_SelVolume_0(BF_PANEL_WINPOS_RIGHT);
				break;			
			case B_F3_KEY:
				NavView();
				break;
			case B_F5_KEY:
				Action_CopyFiles_0();
				break;
			case B_F6_KEY:
				Action_MoveFiles_0();
				break;
			case B_F7_KEY:
				Action_MakeDir_0();
				break;			
			case B_F8_KEY:
				Action_DeleteFiles_0(BF_FILEPANEL_REMOVE_TO_TRASH);
				break;
			case B_F10_KEY:{	
				BMessage	oMessage(BF_MSG_MAIINVIEW_ASK_QUIT);
				BMessenger 	oMessenger(poWinView);
				oMessenger.SendMessage(&oMessage);
				break;}
			}
		}
	}else					
	if(numBytes==1 && (iKeysModifiers & B_RIGHT_OPTION_KEY)){
		BF_Dialog_Alert_Sep("info","key pressed",NULL,this);
	}else			
	if(numBytes==1 && bytes[0]=='*'){
		NavSelectAll_0(bytes[0]);
	}else		
	if(numBytes==1 && bytes[0]=='+'){
		NavSelectAll_0(bytes[0]);
	}else			
	if(numBytes==1 && bytes[0]=='-'){
		NavSelectAll_0(bytes[0]);
	}else			
	if(numBytes==1 && bytes[0]=='\\'){
		NavGoRoot();
	}else{			
		printf("uknown key down\n");
		BView::KeyDown(bytes,numBytes);	
	}
}

bool
BF_GUI_FilePanel::NavSelect(bool b_CursorGoNext)
{
	BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
	if(!poNode) return(false);
	poNode->SetSelect(BF_NODE_SELECT_INVERT);		
	if(b_CursorGoNext && iNavCursorIndex<(loNode.CountItems()-1))	NavDown();	else DrawNodeAt(iNavCursorIndex);
	DrawSelListInfo(true);
	return  true;
}

bool
BF_GUI_FilePanel::NavToStart()
{
	if(0==iNavCursorIndex) return(false );
	NavChangeCursor(0);
	return(true);
}

bool
BF_GUI_FilePanel::NavToEnd()
{
	if((loNode.CountItems()-1)==iNavCursorIndex) return(false );
	NavChangeCursor(loNode.CountItems()-1);
	return(true);
}

void
BF_GUI_FilePanel::NavView()
{
	BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
	if(!poNode) return;
	//	
	switch(poNode->iType){
	case BF_NODE_TYPE_FILE:
		BRect oRect(poWinView->Bounds());
		oRect.bottom-=fNodeHeight;
		/* send message to main window */
		BF_GUI_Func_PanelsShow(false);
		/* make viewer */
		BString s;
		s<<oPath.Path();
		s<<"/";
		s<<poNode->sName;
		BF_GUI_Viewer *poViewer=new BF_GUI_Viewer(oRect,s.String(),this);
		poWinView->AddChild(poViewer);
		poViewer->MakeFocus();
	}
}

bool 
BF_GUI_FilePanel::NavEnter()
{
	BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
	if(!poNode) return(false);
	
	/* if this node link to dir */
	if(poNode->bLink && BF_NODE_TYPE_DIR==poNode->iType){
		BL_String s,s1;
		s<<oPath.Path();
		s<<"/";
		s<<poNode->sName;
		status_t  uRes = BF_Roster_TraverseLink(s.String(),s1);
		/* check result */
		if(uRes!=B_OK){
			/* ups...error */
			BL_System_TranslError(uRes,s1);			
			s="error : ";
			s<<s1;
			BF_Dialog_Alert_Sep("Error","Sorry, can`t enter in this link",NULL,this,s.String());
			return false;
		}else{
			/* result is ok */
			BF_Path oNewPath(s1.String());
			NavGoPath(oNewPath);
			return true;
		}
	}
	
	switch(poNode->iType){
	/* go to parent dir */
	case BF_NODE_TYPE_PARENTDIR:{
		NavGoParentDir();
		break;}
	/* go to select dir */
	case BF_NODE_TYPE_DIR:{
		BF_Path oNewPath = oPath;
		if(!oNewPath.GoLocalDir(poNode->sName)) return(true);
		CashNode_Store(loNode);
		NavGoPath(oNewPath,NULL,true);
		break;}		
	default:{
		if(!BF_Roster_RunFile(oPath.Path(),poNode->Name())){
			BF_Node *poInfo = BF_Roster_LoadNode(oPath.Path(),poNode->Name(),BF_ROSTER_LOAD_TYPE|BF_ROSTER_LOAD_LICON);
			if(!poInfo){
				Action_Alert("Error","I can`t run this file");
				break;				
			}
			/* check mime-type */
			if(poInfo->sType=="" && Action_SetMIME(poInfo,&oPath) && BF_Roster_RunFile(oPath.Path(),poNode->Name())){				
				break;
			}else{
				Action_Alert("Error","I can`t run this file");
			}
			delete poInfo;
		}
		break;}
	}
	return(true);
}

void 
BF_GUI_FilePanel::Action_Alert(const char *pc_Title,const char *pc_Text)
{
	/* disable panels */
	/* start thread */
	BF_Dialog_Alert_Sep(pc_Title,pc_Text,NULL,this);
		
}

void 
BF_GUI_FilePanel::NavGoParentDir()
{
	BF_Path oNewPath = oPath;
	BString sDir;
	bool	bRet = poSysSetup->iMainStyle & BF_SETUP_MAIN_RETURN_REMEMBER;
	if(bRet) oNewPath.LastDir(sDir);
	while(TRUE){
		if(!oNewPath.GoParent()) return;
		NavGoPath(oNewPath,bRet?&sDir:NULL,!CashNode_Load(loNode));
		break;
	}
}

void 
BF_GUI_FilePanel::NavGoPath(
	const BF_Path & o_NewPath,
	BString *ps_FocusItem,
	bool b_LoadNodes,
	bool b_AddToHistory)
{	
	/* add this path to history */
	if(b_AddToHistory && !oPath.IsEmpty()) PathHistory_Add(&oPath);
	/* stop old monitoring */
	BF_Roster_StopAllMonitors(this);
	/* set new path */
	oPath = o_NewPath;	
	SetHeader(oPath.Path(),(poSysSetup->iMainStyle & BF_SETUP_MAIN_FP_HEADERICON)?oPath.LoadSIcon():NULL);
	/*           */
	if(b_LoadNodes)	MakeNodesList();
	if(ps_FocusItem)	NavChangeCursor(loNode.IndexByName(ps_FocusItem->String()));
	Invalidate();
	/* start monitoring */
	BF_Roster_StartMonitor_Dir(oPath,this);	
	BF_Roster_StartMonitor_Nodes(loNode,this);
}

bool 
BF_GUI_FilePanel::NavDown()
{
	/* check limit */
	if((iNavCursorIndex+1)>=loNode.CountItems()) return false;
	
	int iNewCursor = iNavCursorIndex+1;
	/* check finishing screen */
	if(iNewCursor >= (iNavFirstIndex+iNodesOnScreen)){	
		switch(iMode){
		case BF_FILEPANEL_MODE_COLS:{
			DrawMove_ColsDown();			
			return true;
			break;}
		case BF_FILEPANEL_MODE_FULL:{
			DrawMove_FullDown();			
			return true;
			break;}
		}
	}
	/* go to new pos */
	NavChangeCursor(iNewCursor);
	
	return true;			
}

bool
BF_GUI_FilePanel::NavUp()
{
	/* check for first node */
	if(0==iNavCursorIndex) return false;
	
	if(iNavCursorIndex == iNavFirstIndex){
		switch(iMode){
		case BF_FILEPANEL_MODE_COLS:
			DrawMove_ColsUp();
			return true;
			break;
		case BF_FILEPANEL_MODE_FULL:{
			DrawMove_FullUp();			
			return true;
			break;}
		}
	}	
	NavChangeCursor(iNavCursorIndex-1);
	return true;			
}

bool
BF_GUI_FilePanel::NavRight()
{
	if(iNavCursorIndex == (loNode.CountItems()-1))  return false;
	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{
		if (iColsColCount==1) break;
		int iNewCursor = iNavCursorIndex+iColsNodesInCol;
		if(iNewCursor>loNode.CountItems()) break;
		if(iNewCursor< (iNavFirstIndex + iColsNodesInCol * iColsColCount)) break;
		if( (iNewCursor+iColsNodesInCol-1)>loNode.CountItems()) break;
				
		DrawMove_ColsRight();
		
		return true;								
		break;}
	}
	NavChangeCursor(iNavCursorIndex+iColsNodesInCol);
	return true;
}


bool
BF_GUI_FilePanel::NavLeft()
{
	if(iNavCursorIndex == 0)  return false;

	switch(iMode){
	case BF_FILEPANEL_MODE_COLS:{
		if (iColsColCount==1) break;
		int32 iNewCursor = iNavCursorIndex - iColsNodesInCol;
		if(iNewCursor<0 || ((iNewCursor-iColsNodesInCol)<0) )break;				
		if( iNewCursor >= iNavFirstIndex) break;
		if( (iNewCursor-(int32)iColsNodesInCol)<0) break;
		
		DrawMove_ColsLeft();
		
		return true;								
		break;}
	}
	NavChangeCursor(iNavCursorIndex - iColsNodesInCol);
	return true;
}

bool
BF_GUI_FilePanel::NavPgDown()
{
	/* check for last node */
	if(iNavCursorIndex == (loNode.CountItems()-1))  return false;

	NavChangeCursor(iNavCursorIndex + iNodesOnScreen );			
	
	return true;			
}

bool
BF_GUI_FilePanel::NavPgUp()
{
	/* check for first node */
	if(iNavCursorIndex == 0)  return false;
	/* if all list showing  */
	if(loNode.CountItems()<=iNodesOnScreen){
		NavChangeCursor(0);	
	}else{		
		iNavCursorIndex -= iNodesOnScreen;
		iNavFirstIndex -= iNodesOnScreen;
		if(iNavCursorIndex<0)	iNavCursorIndex = 0;
		if(iNavFirstIndex<0)	iNavFirstIndex = 0;
		DrawNodes(true);	
	}	
	return true;			
}


bool 
BF_GUI_FilePanel::NavGoRoot()
{
	BF_Path oPath;
	oPath.Set("/boot");
	CashNode_Clear();
	NavGoPath(oPath);
		
	return(true);
}

bool 
BF_GUI_FilePanel::NavChangeCursor(int i_NewCursor,bool b_DrawNodes,bool b_ChangeVars)
{
	if(b_ChangeVars){
		if(i_NewCursor<0) i_NewCursor = 0;
		if(i_NewCursor>=loNode.CountItems()) i_NewCursor = loNode.CountItems()-1;
		///
		/* if go up */
		if(i_NewCursor<iNavFirstIndex){
			iNavFirstIndex = i_NewCursor;
			iNavCursorIndex = i_NewCursor;	
			if(b_DrawNodes) 	DrawNodes(true);
		}else
		/* if go down */	
		if((i_NewCursor - iNavFirstIndex+1)>iNodesOnScreen){
			iNavFirstIndex = i_NewCursor - iNodesOnScreen+1;		
			iNavCursorIndex = i_NewCursor;
			if(b_DrawNodes)	DrawNodes(true);
		}else
		/* change on screen */
		{
			if(b_DrawNodes){		
				BRect 	oRect,oRect1,oRect2;
				BPoint	oPoint;
				char	pc[500];
				bool	bFocused=IsFocus() || bFixedFocus;
				
				if(bFocused){
					DrawNode(iNavCursorIndex,false,oPoint,oRect,pc,&oRect1,true);				
					DrawNode(i_NewCursor,true,oPoint,oRect,pc,&oRect2,true);
				}
			}
			iNavCursorIndex = i_NewCursor;	
		}
	}
	DrawFileInfo();
	if(poInfoPanel){
		poInfoPanel->SetNode(loNode.NodeAt(iNavCursorIndex),oPath);
	}
	return true;
}

void 					
BF_GUI_FilePanel::OnFocus(bool b_Focused)
{
	if(!bFixedFocus){		
		/* redraw focus_node */
		BRect 	oRect,oRect1;
		BPoint	oPoint;
		char	pc[500];
		DrawNode(iNavCursorIndex,b_Focused,oPoint,oRect,pc,&oRect1,true);	
	}
	/* call parent */
	BF_GUI_Panel::OnFocus(b_Focused);
	
	if(b_Focused){
		/* set keyboard shortcats */
		poWin->AddShortcut('s',B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_GIVE_SELFPATH_TO_FRIEND),this);
		poWin->AddShortcut(B_BACKSPACE,B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_HISTORY_BACK),this);		
		poWin->AddShortcut(B_LEFT_ARROW,B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_HISTORY_BACK),this);		
		for(int i=0;i<10;i++){
			BMessage *poMessage = new BMessage(BF_MSG_FILEPANEL_COLSMODE_SET_COLSCOUNT);
			poMessage->AddInt32("iCount",(int32)i);
			poWin->AddShortcut((char)('0'+i),B_COMMAND_KEY,poMessage,this);		
		}
		poWin->AddShortcut('p',B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_SHOW_ICONS),this);		
		poWin->AddShortcut('i',B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_SHOW_INFOPANEL),this);
		poWin->AddShortcut('a',B_COMMAND_KEY,new BMessage(BF_MSG_FILEPANEL_SHOW_ATTRDIALOG),this);		
		poWin->AddShortcut('q',B_COMMAND_KEY,new BMessage(BF_MSG_MAIINVIEW_ASK_QUIT),poWinView);				
	}else{		
		PulseAction_Remove(BF_FILEPANEL_PULSE_ALL);
		/* clear keyboard shortcats */
		poWin->RemoveShortcut('s',B_COMMAND_KEY);
		poWin->RemoveShortcut(B_BACKSPACE,B_COMMAND_KEY);		
		poWin->RemoveShortcut(B_LEFT_ARROW,B_COMMAND_KEY);		
		for(int i=0;i<10;i++){			
			poWin->RemoveShortcut((char)('1'+i),B_COMMAND_KEY);
		}
		poWin->RemoveShortcut('p',B_COMMAND_KEY);
		poWin->RemoveShortcut('i',B_COMMAND_KEY);
		poWin->RemoveShortcut('a',B_COMMAND_KEY);
	}		
	/* check betatesting status */
	if(BF_BETATEST_ASK_KEY==iBetaTestingStatus){
		iBetaTestingStatus = BF_BETATEST_CHECKING_KEY;
		BF_BetaTesting_CheckKey_Start(this);
	}
}


void					
BF_GUI_FilePanel::FrameResized(float width, float height)
{
	Calc();
	BF_GUI_Panel::FrameResized(width,height);
}

void
BF_GUI_FilePanel::MessageReceived(BMessage* po_Message)
{
	switch(po_Message->what) {
	/*=======================*/	
	case B_COPY:{
		BL_String sName;
		sName = oPath.Path();
		sName<<"/";
		BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
		if(poNode)  sName<<poNode->Name();		
		if(be_clipboard->Lock()){				
			printf("be_clipboard->Clear();\n");
			be_clipboard->Clear();
			
			BMessage *po = be_clipboard->Data();
			po->AddData("text/plain", B_MIME_TYPE, sName.String(),sName.Length());
			
			be_clipboard->Commit();
			be_clipboard->Unlock();
		}
		break;}		
	/*=======================*/			
	case BF_MSG_FILEPANEL_SORT:{
		BF_NodeList_SortType iSort;
		iSort = (BF_NodeList_SortType)po_Message->FindInt32("bf_sorttype");
		ChangeSortType(iSort);
		break;
		};
	case BF_MSG_FILEPANEL_SHOW_INFOPANEL:
		ShowInfoPanel();
		break;
	case BF_MSG_FILEPANEL_SHOW_ATTRDIALOG:{
		BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
		if(!poNode) break;
		BF_GUI_AttrDialog_Run(oPath,*poNode,this);
		break;}
	case BF_MSG_FILEPANEL_BETATEST_INPUTPASSWORD:{
		char *pcValue;
		ASSERT(B_OK==po_Message->FindString("edit",(const char**)&pcValue) && pcValue);
		BF_BetaTesting_CheckKey_Finish(pcValue);
		BF_GUI_Func_PanelsEnable(true);
		MakeFocus();
		break;}
	case BF_MSG_FILEPANEL_SHOW_ICONS:{
		switch(iMode){
		case BF_FILEPANEL_MODE_COLS:{
			if(ModeShowSIcon()) 	iModeColsStyles = iModeColsStyles & (~BF_FILEPANEL_COLS_SICON);
			else					iModeColsStyles = iModeColsStyles | BF_FILEPANEL_COLS_SICON;
			break;}
		case BF_FILEPANEL_MODE_FULL:{
			if(ModeShowSIcon()) 	iModeFullStyles = iModeFullStyles & (~BF_FILEPANEL_FULL_SICON);
			else					iModeFullStyles = iModeFullStyles | BF_FILEPANEL_FULL_SICON;
			break;}			
		}						
		Calc();
		MakeNodesList();
		Invalidate();
		break;}
	case BF_MSG_FILEPANEL_HISTORY_BACK:
		PathHistory_GoBack();
		break;
	case BF_MSG_FILEPANEL_GIVE_SELFPATH_TO_FRIEND:{
		poFriend->ChangePath(oPath);
		break;}
	case BF_MSG_FILEPANEL_COLSMODE_SET_COLSCOUNT:{
		int32 iCount=0;
		ASSERT(B_OK==po_Message->FindInt32("iCount",&iCount) && iCount>=0 && iCount<10);
		if( iColsColCount == (uint32)iCount) break;
		switch(iMode){
		case BF_FILEPANEL_MODE_FULL:{		
			bool bIconLoaded = ModeShowSIcon();
			if(iCount>0)	iMode = BF_FILEPANEL_MODE_COLS; else break;
			if(!bIconLoaded && ModeShowSIcon()) MakeNodesList();
			iColsColCount = iCount;
			Calc();
			Invalidate();		
			break;}		
		case BF_FILEPANEL_MODE_COLS:{
			bool bIconLoaded = ModeShowSIcon();
			if(iCount==0)	iMode = BF_FILEPANEL_MODE_FULL; 
			if(!bIconLoaded && ModeShowSIcon()) MakeNodesList();
			iColsColCount = iCount;
			Calc();
			Invalidate();
			break;}
		}
		break;}
	case BF_MSG_FILEPANEL_DO_ENABLE:	
		/* enable panels */
		BF_GUI_Func_PanelsEnable(true);
		MakeFocus();
		break;
	/*=======================*/		
	case BF_MSG_FILEPANEL_MAKELINK_START:
		Action_MakeLink_0();
		break;
	case BF_MSG_MAKELINK:		
		Action_MakeLink_1(po_Message);
		break;
	case BF_MSG_MAKELINKS_FINISH:		
		Action_MakeLink_2(po_Message);
		break;		
	/*=======================*/	
	case BF_MSG_FILEPANEL_COPY_START:
		Action_CopyFiles_0();
		break;
	case BF_MSG_FILEPANEL_COPY_1:
		Action_CopyFiles_1(po_Message);
		break;
	case BF_MSG_FILEPANEL_COPY_FINISHED:
		Action_CopyFiles_2(po_Message);
		break;		
	/*=======================*/
	case BF_MSG_FILEPANEL_MOVE_START:
		Action_MoveFiles_0();
		break;
	case BF_MSG_MOVE:{
		Action_MoveFiles_1(po_Message);
		break;}
	case BF_MSG_MOVE_FINISHED:
		Action_MoveFiles_2(po_Message);
		break;
	/*=======================*/		
	case BF_MSG_FILEPANEL_MAKEDIR_START:
		Action_MakeDir_0();
		break;
	case BF_MSG_MAKEDIR:{
		Action_MakeDir_1(po_Message);
		break;}
	/*=======================*/		
	case BF_MSG_FILEPANEL_MOVETOVOID_START:
		Action_DeleteFiles_0(BF_FILEPANEL_REMOVE_TO_VOID);
		break;
	case BF_MSG_FILEPANEL_MOVETOTRASH_START:
		Action_DeleteFiles_0(BF_FILEPANEL_REMOVE_TO_TRASH);
		break;
	case BF_MSG_DELETE:{
		Action_DeleteFiles_1(po_Message);
		break;}
	case BF_MSG_DELETE_FINISH:
		Action_DeleteFiles_2(po_Message);
		break;
	/*=======================*/		
	/* message from Viewer */
	case BF_MSG_FILEPANEL_VIEWER_START:{
		NavView();
		break;}
	case BF_MSG_VIEWER_CLOSED:{
		/* send message to main window */
		{
			BF_GUI_Func_PanelsShow(true);
			MakeFocus();
			PrepareKeysMenu();
		}
		break;}	
	/*=======================*/			
	case BF_MSG_FILEPANEL_VOL_LEFT_START:{
		Action_SelVolume_0(BF_PANEL_WINPOS_LEFT);
		break;}
	case BF_MSG_FILEPANEL_VOL_RIGHT_START:{
		Action_SelVolume_0(BF_PANEL_WINPOS_RIGHT);
		break;}		
	case BF_MSG_FILEPANEL_VOL_SELECTED:{			
		Action_SelVolume_1(po_Message);
		break;}
	/*=======================*/				
	case BF_MSG_INPUTMASK:{
		NavSelectAll_1(po_Message);
		break;};
	/* total app messages */
	/* keyboard modifiers changed */
	case 1599425348:
	case 1599425365:
		PrepareKeysMenu();
		break;
	/* main window  receive focus */
	case BF_MSG_TOTAL_WIN_RECEIVE_FOCUS:
		PrepareKeysMenu();
		break;
	/* command to add new node_list */
	case BF_MSG_PANEL_ADD_NODES:{
		BF_NodeCollection *ploNode=NULL;
		ASSERT(B_OK==po_Message->FindPointer("bf_ploNodeList",(void**)&ploNode)
			&& ploNode);
		Nodes_AddList(ploNode);
		break;}
	case B_NODE_MONITOR:{
		//printf("monitoring message %i\n",po_Message->what);
		Nodes_HandleEventFromMonitor(po_Message);
		break;}
	default:{
		//printf("uknown message %i\n",po_Message->what);
		BF_GUI_Panel::MessageReceived(po_Message);
		break;}  	
	}
}


void 
BF_GUI_FilePanel::CashNode_Store(BF_NodeList & lo_Node)
{
	return; //!!!!!!! 
	
	BF_NodeList *plo = new BF_NodeList();
	plo->AddList(&lo_Node);
	lo_Node.MakeEmpty();
	loCashNode.AddItem(plo);
}

bool
BF_GUI_FilePanel::CashNode_Load(BF_NodeList & lo_Node)
{
	if(loCashNode.CountItems()==0) return false;

	lo_Node.DeleteItems();	
	
	BF_NodeList* ploNode = (BF_NodeList*)
		loCashNode.ItemAt(loCashNode.CountItems()-1);
		
	ASSERT(ploNode);
	loCashNode.RemoveItem(ploNode);	
	lo_Node.AddList(ploNode);
	ploNode->MakeEmpty();
	delete ploNode;
	
	return true;
}

void
BF_GUI_FilePanel::CashNode_Clear()
{
	loCashNode.DeleteItems();
}

void
BF_GUI_FilePanel::PathHistory_Add(BF_Path * po_Path)
{
	if(lsPathHistory.CountItems()==poSysSetup->iPathHistorySize) lsPathHistory.DeleteItemAt(0);
	ASSERT(po_Path);
	lsPathHistory.AddItem(new BL_String(po_Path->Path()));	
}

void					
BF_GUI_FilePanel::PathHistory_GoBack()
{
	BL_String * ps = (BL_String*)lsPathHistory.ItemAt(lsPathHistory.CountItems()-1);
	/* check existing of item */
	if(!ps){
		BF_Dialog_Alert_Sep("WARNING","sorry, but path_history is empty",NULL,this);
		return;
	}
	BF_Path oNewPath(ps->String());
	lsPathHistory.DeleteItemAt(lsPathHistory.CountItems()-1);
	NavGoPath(oNewPath,NULL,true,false);
}

void					
BF_GUI_FilePanel::Nodes_AddList(BF_NodeCollection *plo_Node)
{
	ASSERT(plo_Node);
	if(Enabled()){
		/* check for set_cursor */		
		BF_Node *po=NULL;
		for(int i=0;i<plo_Node->CountItems();i++){
			po = plo_Node->NodeAt(i);
			if(loNodeEvent.EventByNameType(po->sName.String(),BF_NODE_EVENT_SETCURSOR))	break;
			po = NULL;
		}
		/* add list to panel */
		loNode.AddList(plo_Node);
		loNode.Sort(iSortType,iSortStyle);
		DrawNodes(true);
		DrawListInfo(true);
		if(po) NavChangeCursor(loNode.IndexOf(po));
	}else{
		/* store list in NodeEvents */
		loNodeEvent.NewEvents(plo_Node,BF_NODE_EVENT_ADDED);
	}
	/* delete temp collection */
	BF_Roster_StartMonitor_Nodes(*plo_Node,this);
	delete plo_Node;
}

void					
BF_GUI_FilePanel::Nodes_RefreshList(BF_NodeCollection *plo_Node)
{
	ASSERT(plo_Node);
	if(Enabled()){
		/* refresh list to panel */		
		loNode.Sort(iSortType,iSortStyle);
		DrawNodes(true);
		DrawSelListInfo(true);
		DrawListInfo(true);
		NavChangeCursor(iNavCursorIndex);
	}else{
		/* store list in NodeEvents */
		loNodeEvent.NewEvents(plo_Node,BF_NODE_EVENT_CHANGED);
	}
	/* delete temp collection */
	delete plo_Node;
}

void
BF_GUI_FilePanel::Nodes_RemoveList(BF_NodeCollection*  plo_Node)
{
	ASSERT(plo_Node);
	/* stop monitoring this nodes */
	BF_Roster_StopMonitor_Nodes(*plo_Node,this);
	/* remove node from node_list */
	loNode.RemoveList(*plo_Node); 
	printf("pre remove nodes from screen \n");
	if(Enabled()){		
		/* remove nodes from screen */
		printf("remove nodes from screen \n");
		DrawNodes(true);
		DrawListInfo(true);
		DrawSelListInfo(true);
		NavChangeCursor(iNavCursorIndex);			
		plo_Node->DeleteItems();
	}else{
		/* store to events */
		loNodeEvent.NewEvents(plo_Node,BF_NODE_EVENT_REMOVED);
	}	
	delete plo_Node;
}


BF_NodeCollection*
BF_GUI_FilePanel::Nodes_GetSelecting()
{
	BF_NodeCollection *ploSelNode = new BF_NodeCollection();
	loNode.GetSelecting(*ploSelNode);
	if(ploSelNode->CountItems()==0){
		BF_Node *poNode = loNode.NodeAt(iNavCursorIndex);
		if(!poNode) return(FALSE);		
		ploSelNode->AddItem(poNode);
	}
	return ploSelNode;
}


void
BF_GUI_FilePanel::Nodes_HandleEventFromMonitor(BMessage *po_Message)
{
	int32 iOpCode=0;
	if(B_OK!=po_Message->FindInt32("opcode",&iOpCode)) return;		
	
	/* get node_ref */ 
	node_ref uRef;
	po_Message->FindInt32("device", &uRef.device); 
	po_Message->FindInt64("node", &uRef.node); 	
	//
	switch(iOpCode){
	case B_ENTRY_MOVED:{				
		printf("B_ENTRY_MOVED{\n");
		/* get source and destination dirs*/
		node_ref uRefSrcDir,uRefDestDir;
		uRefSrcDir.device = uRef.device;
		if(B_OK!=po_Message->FindInt64("from directory",&uRefSrcDir.node)) break;
		if(B_OK!=po_Message->FindInt64("to directory",&uRefDestDir.node)) break;
		if(B_OK!=po_Message->FindInt32("device",&uRefDestDir.device)) break;
		/* check current dir */
		/*printf("current path device=%i node=%i\n",oPath.NodeRef().device,oPath.NodeRef().node);
		printf("source path device=%i node=%i\n",uRefSrcDir.device,uRefSrcDir.node);
		printf("node path device=%i node=%i\n",uRef.device,uRef.node);*/
		if(oPath.CheckNodeRef(uRefSrcDir)){
			/* remove this node from current list */
			//printf(">try to find node(device=%i node=%i) in this Panel\n",uRef.device,uRef.node);			
			BF_Node *poNode = loNode.NodeByRef(uRef);			
			bool bNodeOnScreen = poNode!=NULL;
			if(!bNodeOnScreen)  poNode = loNodeEvent.NodeByRef(uRef);
			if(poNode){
				/* if renaming */
				if(oPath.CheckNodeRef(uRefDestDir)){
					printf(">>entry_moved is renaming\n");
					//BF_Roster_GetNodeInfoByRef(oPpoNode,
					char *pcName;
					ASSERT(B_OK==po_Message->FindString("name",(const char**)&pcName) && pcName);
					poNode->sName=pcName;
					/* refresh node_information for screen_nodes*/
					if(bNodeOnScreen){
						BF_NodeCollection_New(ploNode,oPath);
						ploNode->AddItem(poNode);
						Nodes_RefreshList(ploNode);
					}
				}else{
				/* if moving */
					printf(">>entry_moved finded node\n");					
					/* add this node to oposite panel */
					BF_Path oFriendPath(poFriend->Path());
					if(oFriendPath.CheckNodeRef(uRefDestDir)){
						printf(">>>oposite path is ok\n");
						poFriend->AddNode(new BF_Node(poNode,true));
					}else{
						printf(">>>oposite path is not ok\n");
						//node_ref uRef = oFriendPath.NodeRef();
						/*printf(">>>current oposite device=%i node=%i\n",
							uRef.device,uRef.node);
						printf(">>>file moved to device=%i node=%i\n",
							uRefDestDir.device,uRefDestDir.node);*/
					};
					/* remove nodes */
					{
						BF_NodeCollection_New(ploNode,oPath);
						ploNode->AddItem(poNode);
						Nodes_RemoveList(ploNode);
					}
					//delete poNode;			
				}
			}else{
				printf(">can`t find this node \n");
			}
		}else
		/* current dir moved */
		if(oPath.CheckNodeRef(uRef)){
			printf("CURRENT DIR REMOVED\n");			
			NavGoParentDir();
		}
		printf("}B_ENTRY_MOVED\n");
		/* */
		break;};
	case B_STAT_CHANGED:{
		printf("B_STAT_CHANGED\n");
		/* seek this node in current node_list */
		BF_Node *poNode = loNode.NodeByRef(uRef);
		bool 	bNodeOnScreen=true;
		if(!poNode){
			poNode = loNodeEvent.NodeByRef(uRef);
			bNodeOnScreen = false;
		}
		if(!poNode){
			printf("B_STAT_CHANGED >>> node not founded\n");
			return;		
		}
		printf("B_STAT_CHANGED >>> node founded\n");		
		if(!BF_Roster_ReloadNode(&oPath,poNode,BF_ROSTER_LOAD_NORMAL)){
			/* can`t update information about this node */
			BF_NodeCollection_New(ploNode,oPath);
			ploNode->AddItem(poNode);
			Nodes_RemoveList(ploNode);	
			break;			
		}
		//
		if(bNodeOnScreen){
			loNode.Sort(iSortType,iSortStyle);
			DrawNodes(true); // !!!!!! moznon prooptmizirovat 
			DrawListInfo(true);
			DrawSelListInfo(true);
			NavChangeCursor(iNavCursorIndex);
		}
		break;};
	case B_ENTRY_REMOVED:{
		printf("B_ENTRY_REMOVED\n");
		/* seek this node in current node_list */
		BF_Node *poNode = loNode.NodeByRef(uRef);
		//printf("poNode seeking device=%i node=%i\n",uRef.device,uRef.node);
		if(!poNode) return;
		printf("poNode finded\n");
		BF_NodeCollection_New(ploNode,oPath);
		ploNode->AddItem(poNode);
		Nodes_RemoveList(ploNode);
		break;}
	case B_ENTRY_CREATED:{
		char *pcName=NULL;
		ASSERT(B_OK==po_Message->FindString("name",(const char**)&pcName));
		printf("B_ENTRY_CREATED {%s}\n",pcName);
		BF_Node *poNode = loNode.NodeByName(pcName);
		if(!poNode){
			int32 iLoadStyle = BF_ROSTER_LOAD_NORMAL;
			if(ModeShowSIcon())	iLoadStyle = iLoadStyle | BF_ROSTER_LOAD_SICON;
			poNode = BF_Roster_LoadNode(oPath.Path(),pcName,iLoadStyle);
			/*BF_Roster_StartMonitor_Node(poNode,this);*/
			BF_NodeCollection_New(ploNode,oPath);
			ploNode->AddItem(poNode);
			Nodes_AddList(ploNode);
		}
		///
		break;}		
	}
}


bool					
BF_GUI_FilePanel::ModeShowSIcon()
{
	if(BF_FILEPANEL_MODE_FULL==iMode && (iModeFullStyles & BF_FILEPANEL_FULL_SICON)) return true;
	if(BF_FILEPANEL_MODE_COLS==iMode && (iModeColsStyles & BF_FILEPANEL_COLS_SICON)) return true;	
	return false;
}

void					
BF_GUI_FilePanel::ChangePath(const BF_Path & o_Path)
{
	NavGoPath(o_Path);
}

BF_Path&				
BF_GUI_FilePanel::Path()
{ 
	BF_Path *poPath = new BF_Path(oPath);
	return *poPath;
}

void					
BF_GUI_FilePanel::AddNode(BF_Node *po_Node)
{
	ASSERT(po_Node);
	//
	BF_NodeCollection *ploAddNode = new BF_NodeCollection();
	//BF_Roster_StartMonitor_Node(po_Node,this);
	ploAddNode->AddItem(po_Node);
	Nodes_AddList(ploAddNode);
}

void
BF_GUI_FilePanel::ChangeSortType(BF_NodeList_SortType i_NewSortType)
{
	Window()->Lock();
	//
	iSortType = i_NewSortType;
	loNode.Sort(iSortType,iSortStyle);	
	DrawNodes(true);
	//
	Window()->Unlock();
}

void
BF_GUI_FilePanel::ShowInfoPanel()
{
	BF_Node *poCursor = loNode.NodeAt(iNavCursorIndex);
	if(!poCursor) return;
	BView *poParent = Parent();
	if(!poParent || !poFriend) return ;
	
	if(poInfoPanel){
		Window()->Lock();
		poInfoPanel->RemoveSelf();
		delete poInfoPanel;		
		poInfoPanel = NULL;
		poFriend->Show();
		Window()->Unlock();
	}else{
		/* calc rect for new panel */
		BRect oRect;
		oRect = poFriend->Frame();	
		/* make new panel */
		poInfoPanel = new BF_GUI_InfoPanel(oRect,poFriend->iWinPosition);
		poInfoPanel->SetNode(poCursor,oPath);
		
		
		Window()->Lock();
		poParent->AddChild(poInfoPanel);
		poFriend->Hide();
		Window()->Unlock();
	}
}
	



BF_GUI_FilePanel_ActNodeIcon::BF_GUI_FilePanel_ActNodeIcon(
	BRect o_Rect,
	const char *pc_Name,
	uint32	i_FollowMode,
	uint32	i_Flags
):BF_GUI_DialogView(o_Rect,pc_Name,i_FollowMode,i_Flags|B_WILL_DRAW)
{
	SetViewColor(B_TRANSPARENT_COLOR);	
	SetFont(&poSysSetup->oFontToolView.oFont);
	poNode = NULL;
}

void
BF_GUI_FilePanel_ActNodeIcon::SetNode(BF_Node *po_Node)
{
	poNode = po_Node;
	Draw(Bounds());
}

void					
BF_GUI_FilePanel_ActNodeIcon::Draw(BRect o_Rect)
{
	BRect	oRect;
	BPoint 	oPoint(0,0);
	/* draw back */
	oRect = Bounds();
	SetHighColor(poSysSetup->aColor[BF_COLOR_DIALOG_BACK]);
	FillRect(oRect);
	
	/* draw text and icon */
	if(poNode){
		float fWidth = poSysSetup->oFontToolView.oFont.StringWidth(poNode->sName.String());
		if(poNode->poSIcon) fWidth+=20;		
		oPoint.Set(oRect.Width()/2-fWidth/2,0);
		
		/* draw icon */
		if(poNode->poSIcon){
			SetDrawingMode(B_OP_ALPHA);				
			DrawBitmap(poNode->poSIcon,oPoint);
			SetDrawingMode(B_OP_COPY);
			oPoint.x+=20;
		}
		/* draw text */		
		oPoint.y = poSysSetup->oFontToolView.fAscent;
		SetHighColor(poSysSetup->aColor[BF_COLOR_DIALOG_BUTTON_TEXT]);
		SetLowColor(poSysSetup->aColor[BF_COLOR_DIALOG_BACK]);
		DrawString(poNode->sName.String(),oPoint);			
	}
	
}
/////////////////////////////////////////////////////////////////////////////////////////////
void
BF_GUI_FilePanel::PulseAction_Add(int32 i_NewActions)
{
	iPulseActions = iPulseActions | i_NewActions;
	
	if(! ( Flags() & B_PULSE_NEEDED )){
		SetFlags( Flags() | B_PULSE_NEEDED);
		Window()->Lock();
		Window()->SetPulseRate(10);
		Window()->Unlock();
	}	
}

void					
BF_GUI_FilePanel::Pulse(void)
{
	if(iPulseActions & BF_FILEPANEL_PULSE_MOVEDOWN)	NavDown();
	if(iPulseActions & BF_FILEPANEL_PULSE_MOVEUP)	NavUp();	
}

void
BF_GUI_FilePanel::PulseAction_Remove(int32 i_Actions)
{
	if(iPulseActions==BF_FILEPANEL_PULSE_NONE) return;
	iPulseActions = iPulseActions & (!i_Actions);
	if(iPulseActions==BF_FILEPANEL_PULSE_NONE){
		SetFlags( Flags() & (!B_PULSE_NEEDED));		
	}
}
/////////////////////////////////////////////////////////////////////////////////////////////